searchNodes=[{"ref":"ExHal.html","title":"ExHal","module":"ExHal","type":"module","doc":"Use HAL APIs with ease. Given a resource http://example.com/hal whose HAL representation looks like { &quot;name&quot;: &quot;Hello!&quot;, &quot;_links&quot;: { &quot;self&quot; : { &quot;href&quot;: &quot;http://example.com&quot; }, &quot;profile&quot;: [{ &quot;href&quot;: &quot;http://example.com/special&quot; }, { &quot;href&quot;: &quot;http://example.com/normal&quot; }] } } iex&gt; {:ok, doc, response_header} = ExHal.client ...&gt; |&gt; ExHal.Client.add_headers(&quot;User-Agent&quot;: &quot;MyClient/1.0&quot;) ...&gt; |&gt; ExHal.Client.get(&quot;http://example.com/hal&quot;) %ExHal.Document{...} Now we have an entry point to the API we can follow links to navigate around. iex&gt; ExHal.follow_link(doc, &quot;profile&quot;) {:ok, %ExHal.Document{...}, %ExHal.ResponseHeader{...}} iex&gt; ExHal.follow_link(doc, &quot;self&quot;) {:ok, %ExHal.Document{...}, %ExHal.ResponseHeader{...}} iex&gt; ExHal.follow_links(doc, &quot;profile&quot;) [{:ok, %ExHal.Document{...}, %ExHal.ResponseHeader{...}}, {:ok, %ExHal.Document{...}, %ExHal.ResponseHeader{...}}] We can specify headers for each request in addition to the headers specified in the client. iex&gt; ExHal.follow_links(doc, &quot;profile&quot;, headers: [&quot;Accept&quot;: &quot;application/vnd.custom.json+type&quot;]) [{:ok, %ExHal.Document{...}, %ExHal.ResponseHeader{...}}, {:ok, %ExHal.Document{...}, %ExHal.ResponseHeader{...}}] If we try to follow a non-existent or compound link with ExHal.follow_link it will return an error tuple. iex&gt; ExHal.follow_link(doc, &quot;nonexistent&quot;) {:error, %ExHal.Error{reason: &quot;no such link&quot;}} iex&gt; ExHal.follow_link(doc, &quot;profile&quot;, strict: true) {:error, %ExHal.Error{reason: &quot;multiple choices&quot;}} If we try to follow a non-existent with ExHal.follow_links it will return a list of error tuples. iex&gt; ExHal.follow_links(doc, &quot;nonexistent&quot;) [{:error, %ExHal.Error{reason: &quot;no such link&quot;}}] Collections Consider a resource http://example.com/hal-collection whose HAL representation looks like { &quot;_links&quot;: { &quot;self&quot; : { &quot;href&quot;: &quot;http://example.com/hal-collection&quot; }, &quot;item&quot;: [{ &quot;href&quot;: &quot;http://example.com/beginning&quot; }, { &quot;href&quot;: &quot;http://example.com/middle&quot; }] &quot;next&quot;: { &quot;href&quot;: &quot;http://example.com/hal-collection?p=2&quot; } } } and a resource http://example.com/hal-collection?p=2 whose HAL representation looks like { &quot;_links&quot;: { &quot;self&quot; : { &quot;href&quot;: &quot;http://example.com/hal-collection?p=2&quot; }, &quot;item&quot;: [{ &quot;href&quot;: &quot;http://example.com/end&quot; }] } } If we get the first HAL collection resource and turn it into a stream we can use all our favorite Stream functions on it. iex&gt; collection = ExHal.client ...&gt; |&gt; ExHal.Client.add_headers(&quot;User-Agent&quot;: &quot;MyClient/1.0&quot;) ...&gt; |&gt; ExHal.Client.get(&quot;http://example.com/hal-collection&quot;) ...&gt; |&gt; ExHal.to_stream #Function&lt;11.52512309/2 in Stream.resource/3&gt; iex&gt; Stream.map(collection, fn follow_results -&gt; ...&gt; case follow_results do ...&gt; {:ok, a_doc, %ResponseHeader{}} -&gt; ExHal.url(a_doc)} ...&gt; {:error, _} -&gt; :error ...&gt; end ...&gt; end ) [&quot;http://example.com/beginning&quot;, &quot;http://example.com/middle&quot;, &quot;http://example.com/end&quot;]"},{"ref":"ExHal.html#client/0","title":"ExHal.client/0","module":"ExHal","type":"function","doc":"Returns a default client"},{"ref":"ExHal.html#fetch/2","title":"ExHal.fetch/2","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#follow_link/2","title":"ExHal.follow_link/2","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#follow_link/3","title":"ExHal.follow_link/3","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#follow_links/2","title":"ExHal.follow_links/2","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#follow_links/3","title":"ExHal.follow_links/3","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#follow_links/4","title":"ExHal.follow_links/4","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#get_lazy/3","title":"ExHal.get_lazy/3","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#get_links_lazy/3","title":"ExHal.get_links_lazy/3","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#get_property_lazy/3","title":"ExHal.get_property_lazy/3","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#link_target/2","title":"ExHal.link_target/2","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#link_target/3","title":"ExHal.link_target/3","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#link_target_lazy/3","title":"ExHal.link_target_lazy/3","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#link_target_lazy/4","title":"ExHal.link_target_lazy/4","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#link_targets/2","title":"ExHal.link_targets/2","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#link_targets/3","title":"ExHal.link_targets/3","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#link_targets_lazy/3","title":"ExHal.link_targets_lazy/3","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#link_targets_lazy/4","title":"ExHal.link_targets_lazy/4","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#patch/3","title":"ExHal.patch/3","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#patch/4","title":"ExHal.patch/4","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#post/3","title":"ExHal.post/3","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#post/4","title":"ExHal.post/4","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#to_stream/1","title":"ExHal.to_stream/1","module":"ExHal","type":"function","doc":"Returns a stream that yields the items in the rfc 6573 collection represented by a_doc."},{"ref":"ExHal.html#url/1","title":"ExHal.url/1","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.html#url/2","title":"ExHal.url/2","module":"ExHal","type":"function","doc":""},{"ref":"ExHal.Assertions.html","title":"ExHal.Assertions","module":"ExHal.Assertions","type":"module","doc":"Convenience functions for asserting things about HAL documents iex&gt; import ExUnit.Assertions nil iex&gt; import ExHal.Assertions nil iex&gt; assert_property ~s({&quot;name&quot;: &quot;foo&quot;}), &quot;name&quot; true iex&gt; assert_property ~s({&quot;name&quot;: &quot;foo&quot;}), &quot;address&quot; ** (ExUnit.AssertionError) address is absent iex&gt; assert_property ~s({&quot;name&quot;: &quot;foo&quot;}), &quot;name&quot;, eq &quot;foo&quot; true iex&gt; assert_property ~s({&quot;name&quot;: &quot;foo&quot;}), &quot;name&quot;, matches ~r/fo/ true iex&gt; assert_property ~s({&quot;name&quot;: &quot;foo&quot;}), &quot;name&quot;, eq &quot;bar&quot; ** (ExUnit.AssertionError) expected property `name` to eq(&quot;bar&quot;) iex&gt; assert_link_target ~s({&quot;_links&quot;: { &quot;profile&quot;: {&quot;href&quot;: &quot;http://example.com&quot; }}}), ...&gt; &quot;profile&quot; true iex&gt; assert_link_target ~s({&quot;_links&quot;: { &quot;profile&quot;: {&quot;href&quot;: &quot;http://example.com&quot; }}}), ...&gt; &quot;item&quot; ** (ExUnit.AssertionError) link `item` is absent iex&gt; assert_link_target ~s({&quot;_links&quot;: { &quot;profile&quot;: {&quot;href&quot;: &quot;http://example.com&quot; }}}), ...&gt; &quot;profile&quot;, eq &quot;http://example.com&quot; true iex&gt; assert_link_target ~s({&quot;_links&quot;: { &quot;profile&quot;: {&quot;href&quot;: &quot;http://example.com&quot; }}}), ...&gt; &quot;profile&quot;, matches ~r/example.com/ true iex&gt; assert_link_target ~s({&quot;_links&quot;: { &quot;profile&quot;: {&quot;href&quot;: &quot;http://example.com&quot; }}}), ...&gt; &quot;profile&quot;, eq &quot;http://bad.com&quot; ** (ExUnit.AssertionError) expected (at least one) `item` link to eq(&quot;http://bad.com&quot;) but found only http://example.com iex&gt; assert collection(&quot;{}&quot;) |&gt; Enum.empty? true iex&gt; assert 1 == collection(&quot;{}&quot;) |&gt; Enum.count ** (ExUnit.AssertionError) Assertion with == failed"},{"ref":"ExHal.Assertions.html#assert_link_target/2","title":"ExHal.Assertions.assert_link_target/2","module":"ExHal.Assertions","type":"macro","doc":"Asserts that the specifed link exists and that its target checks out."},{"ref":"ExHal.Assertions.html#assert_link_target/3","title":"ExHal.Assertions.assert_link_target/3","module":"ExHal.Assertions","type":"macro","doc":""},{"ref":"ExHal.Assertions.html#assert_property/2","title":"ExHal.Assertions.assert_property/2","module":"ExHal.Assertions","type":"macro","doc":"Asserts that a property exists and, optionally, that its value checks out."},{"ref":"ExHal.Assertions.html#assert_property/3","title":"ExHal.Assertions.assert_property/3","module":"ExHal.Assertions","type":"macro","doc":""},{"ref":"ExHal.Assertions.html#collection/1","title":"ExHal.Assertions.collection/1","module":"ExHal.Assertions","type":"function","doc":"Returns a stream representation of the document."},{"ref":"ExHal.Assertions.html#eq/1","title":"ExHal.Assertions.eq/1","module":"ExHal.Assertions","type":"function","doc":"Returns a function that checks if the actual value is equal to the expected."},{"ref":"ExHal.Assertions.html#matches/1","title":"ExHal.Assertions.matches/1","module":"ExHal.Assertions","type":"function","doc":"Returns a function that checks if the actual value matches the expected pattern."},{"ref":"ExHal.Assertions.html#p_assert_link_target/4","title":"ExHal.Assertions.p_assert_link_target/4","module":"ExHal.Assertions","type":"function","doc":""},{"ref":"ExHal.Assertions.html#p_assert_property/4","title":"ExHal.Assertions.p_assert_property/4","module":"ExHal.Assertions","type":"function","doc":""},{"ref":"ExHal.Client.html","title":"ExHal.Client","module":"ExHal.Client","type":"behaviour","doc":"Behavior related to making HTTP requests. Examples iex&gt; ExHal.Client.new() %ExHal.Client{}"},{"ref":"ExHal.Client.html#add_headers/2","title":"ExHal.Client.add_headers/2","module":"ExHal.Client","type":"function","doc":"Returns client that will include the specified headers in any request made with it."},{"ref":"ExHal.Client.html#delete/3","title":"ExHal.Client.delete/3","module":"ExHal.Client","type":"function","doc":""},{"ref":"ExHal.Client.html#c:delete/3","title":"ExHal.Client.delete/3","module":"ExHal.Client","type":"callback","doc":""},{"ref":"ExHal.Client.html#get/3","title":"ExHal.Client.get/3","module":"ExHal.Client","type":"function","doc":""},{"ref":"ExHal.Client.html#c:get/3","title":"ExHal.Client.get/3","module":"ExHal.Client","type":"callback","doc":""},{"ref":"ExHal.Client.html#new/0","title":"ExHal.Client.new/0","module":"ExHal.Client","type":"function","doc":""},{"ref":"ExHal.Client.html#new/1","title":"ExHal.Client.new/1","module":"ExHal.Client","type":"function","doc":""},{"ref":"ExHal.Client.html#new/2","title":"ExHal.Client.new/2","module":"ExHal.Client","type":"function","doc":"Returns a new client."},{"ref":"ExHal.Client.html#patch/4","title":"ExHal.Client.patch/4","module":"ExHal.Client","type":"function","doc":""},{"ref":"ExHal.Client.html#c:patch/4","title":"ExHal.Client.patch/4","module":"ExHal.Client","type":"callback","doc":""},{"ref":"ExHal.Client.html#post/4","title":"ExHal.Client.post/4","module":"ExHal.Client","type":"function","doc":""},{"ref":"ExHal.Client.html#c:post/4","title":"ExHal.Client.post/4","module":"ExHal.Client","type":"callback","doc":""},{"ref":"ExHal.Client.html#put/4","title":"ExHal.Client.put/4","module":"ExHal.Client","type":"function","doc":""},{"ref":"ExHal.Client.html#c:put/4","title":"ExHal.Client.put/4","module":"ExHal.Client","type":"callback","doc":""},{"ref":"ExHal.Client.html#t:http_response/0","title":"ExHal.Client.http_response/0","module":"ExHal.Client","type":"type","doc":"The return value of any function that makes an HTTP request."},{"ref":"ExHal.Client.html#t:t/0","title":"ExHal.Client.t/0","module":"ExHal.Client","type":"opaque","doc":"Represents a client configuration/connection. Create with new function."},{"ref":"ExHal.Collection.html","title":"ExHal.Collection","module":"ExHal.Collection","type":"module","doc":"Utility functions for dealing with RFC 6473 collections"},{"ref":"ExHal.Collection.html#render!/1","title":"ExHal.Collection.render!/1","module":"ExHal.Collection","type":"function","doc":"Returns a string representation of this HAL collection."},{"ref":"ExHal.Collection.html#to_json_hash/1","title":"ExHal.Collection.to_json_hash/1","module":"ExHal.Collection","type":"function","doc":"Deprecated see render/1"},{"ref":"ExHal.Collection.html#to_stream/1","title":"ExHal.Collection.to_stream/1","module":"ExHal.Collection","type":"function","doc":"Returns a stream that iterate over the collection represented by a_doc."},{"ref":"ExHal.Document.html","title":"ExHal.Document","module":"ExHal.Document","type":"module","doc":"A document is the representaion of a single resource in HAL."},{"ref":"ExHal.Document.html#__struct__/0","title":"ExHal.Document.__struct__/0","module":"ExHal.Document","type":"function","doc":"Returns a new %ExHal.Document representing the HAL document provided."},{"ref":"ExHal.Document.html#fetch/2","title":"ExHal.Document.fetch/2","module":"ExHal.Document","type":"function","doc":"Fetches value of specified property or links whose rel matches Returns {:ok, &lt;property value&gt;} if name identifies a property; `{:ok, [%Link{}, ...]}` if `name` identifies a link; `:error` othewise"},{"ref":"ExHal.Document.html#from_parsed_hal/1","title":"ExHal.Document.from_parsed_hal/1","module":"ExHal.Document","type":"function","doc":"Returns new ExHal.Document"},{"ref":"ExHal.Document.html#from_parsed_hal/2","title":"ExHal.Document.from_parsed_hal/2","module":"ExHal.Document","type":"function","doc":""},{"ref":"ExHal.Document.html#get/3","title":"ExHal.Document.get/3","module":"ExHal.Document","type":"function","doc":"Returns the link or property of the specified name, or default if neither or found."},{"ref":"ExHal.Document.html#get_lazy/3","title":"ExHal.Document.get_lazy/3","module":"ExHal.Document","type":"function","doc":"Returns link or property of the specified name, or the result of default_fun if neither are found."},{"ref":"ExHal.Document.html#get_links/3","title":"ExHal.Document.get_links/3","module":"ExHal.Document","type":"function","doc":"Returns [%Link{}...] when link exists or result of default otherwise."},{"ref":"ExHal.Document.html#get_links_lazy/3","title":"ExHal.Document.get_links_lazy/3","module":"ExHal.Document","type":"function","doc":"Returns [%Link{}...] when link exists or result of default_fun otherwise."},{"ref":"ExHal.Document.html#get_property/3","title":"ExHal.Document.get_property/3","module":"ExHal.Document","type":"function","doc":"Returns property value when property exists or default otherwise"},{"ref":"ExHal.Document.html#get_property_lazy/3","title":"ExHal.Document.get_property_lazy/3","module":"ExHal.Document","type":"function","doc":"Returns &lt;property value&gt; when property exists or result of default_fun otherwise"},{"ref":"ExHal.Document.html#has_link?/2","title":"ExHal.Document.has_link?/2","module":"ExHal.Document","type":"function","doc":"Returns true iff the document contains at least one link with the specified rel."},{"ref":"ExHal.Document.html#parse/2","title":"ExHal.Document.parse/2","module":"ExHal.Document","type":"function","doc":""},{"ref":"ExHal.Document.html#parse!/2","title":"ExHal.Document.parse!/2","module":"ExHal.Document","type":"function","doc":"Returns a new %ExHal.Document representing the HAL document provided."},{"ref":"ExHal.Document.html#put_link/4","title":"ExHal.Document.put_link/4","module":"ExHal.Document","type":"function","doc":"Add a link to a Document. Returns new ExHal.Document with the specified link."},{"ref":"ExHal.Document.html#put_property/3","title":"ExHal.Document.put_property/3","module":"ExHal.Document","type":"function","doc":"Add or update a property to a Document. Returns new ExHal.Document with the specified property set to the specified value."},{"ref":"ExHal.Document.html#render!/1","title":"ExHal.Document.render!/1","module":"ExHal.Document","type":"function","doc":"Returns a string representation of this HAL document."},{"ref":"ExHal.Document.html#to_json_hash/1","title":"ExHal.Document.to_json_hash/1","module":"ExHal.Document","type":"function","doc":"Deprecated See to_json_map/1"},{"ref":"ExHal.Document.html#to_json_map/1","title":"ExHal.Document.to_json_map/1","module":"ExHal.Document","type":"function","doc":"Returns a map that matches the shape of the intended JSON output."},{"ref":"ExHal.Document.html#url/2","title":"ExHal.Document.url/2","module":"ExHal.Document","type":"function","doc":"Returns {:ok, &lt;url of specified document&gt;} or :error."},{"ref":"ExHal.Document.html#t:t/0","title":"ExHal.Document.t/0","module":"ExHal.Document","type":"opaque","doc":""},{"ref":"ExHal.Error.html","title":"ExHal.Error","module":"ExHal.Error","type":"module","doc":""},{"ref":"ExHal.Form.html","title":"ExHal.Form","module":"ExHal.Form","type":"module","doc":"Represents a Dwolla style HAL form. Generally these are acquired from ExHal.Document.get_form/2, ExHal.Document.fetch_form/2, etc"},{"ref":"ExHal.Form.html#client_module/0","title":"ExHal.Form.client_module/0","module":"ExHal.Form","type":"function","doc":""},{"ref":"ExHal.Form.html#from_forms_entry/1","title":"ExHal.Form.from_forms_entry/1","module":"ExHal.Form","type":"function","doc":"Creates a new form from raw form JSON. Raises ArgumentError if a_map is not a valid form fragment."},{"ref":"ExHal.Form.html#get_fields/1","title":"ExHal.Form.get_fields/1","module":"ExHal.Form","type":"function","doc":"Returns list of the fields in this form."},{"ref":"ExHal.Form.html#set_field_value/3","title":"ExHal.Form.set_field_value/3","module":"ExHal.Form","type":"function","doc":"Returns form with the specified fields value updated. Raises ArgumentError if the specified field doesn&#39;t exist."},{"ref":"ExHal.Form.html#submit/2","title":"ExHal.Form.submit/2","module":"ExHal.Form","type":"function","doc":"Submits form and returns the response."},{"ref":"ExHal.Form.html#t:t/0","title":"ExHal.Form.t/0","module":"ExHal.Form","type":"opaque","doc":"A form that can be completed and submitted. This type is opaque and should only be used as an argument to functions in this, Elixir.ExHal.Form module."},{"ref":"ExHal.FormField.html","title":"ExHal.FormField","module":"ExHal.FormField","type":"module","doc":"Functions for working with HAL form fields."},{"ref":"ExHal.FormField.html#from_field_entry/1","title":"ExHal.FormField.from_field_entry/1","module":"ExHal.FormField","type":"function","doc":"Create a new form field from parsed form json. Raises ArgumentError if json is invalid."},{"ref":"ExHal.FormField.html#set_value/2","title":"ExHal.FormField.set_value/2","module":"ExHal.FormField","type":"function","doc":"Returns a form field with the specified value. Raises ArgumentError if the new value is the wrong type."},{"ref":"ExHal.FormField.html#t:field_types/0","title":"ExHal.FormField.field_types/0","module":"ExHal.FormField","type":"type","doc":"Field types."},{"ref":"ExHal.FormField.html#t:field_value/0","title":"ExHal.FormField.field_value/0","module":"ExHal.FormField","type":"type","doc":"Possible value types."},{"ref":"ExHal.FormField.html#t:t/0","title":"ExHal.FormField.t/0","module":"ExHal.FormField","type":"type","doc":"A HAL form field."},{"ref":"ExHal.Interpreter.html","title":"ExHal.Interpreter","module":"ExHal.Interpreter","type":"module","doc":"Helps to build interpters of HAL documents. Given a document like { &quot;name&quot;: &quot;Jane Doe&quot;, &quot;mailingAddress&quot;: &quot;123 Main St&quot;, &quot;_links&quot;: { &quot;app:department&quot;: { &quot;href&quot;: &quot;http://example.com/dept/42&quot; } } } We can define an interpreter for it. defmodule PersonInterpreter do use ExHal.Interpreter defextract :name defextract :address, from: &quot;mailingAddress&quot; defextractlink :department_url, rel: &quot;app:department&quot; end We can use this interpreter to to extract the pertinent parts of the document into a map. iex&gt; PersonInterpreter.to_params(doc) %{name: &quot;Jane Doe&quot;, address: &quot;123 Main St&quot;, department_url: &quot;http://example.com/dept/42&quot;}"},{"ref":"ExHal.Interpreter.html#defextract/2","title":"ExHal.Interpreter.defextract/2","module":"ExHal.Interpreter","type":"macro","doc":"Define a property extractor. name - the name of the parameter to extract options - Keywords of optional arguments :from - the name of the property in the JSON document. Default is to_string(name)."},{"ref":"ExHal.Interpreter.html#defextractlink/2","title":"ExHal.Interpreter.defextractlink/2","module":"ExHal.Interpreter","type":"macro","doc":"Define a link extractor. name - the name of the parameter to extract options - Keywords of optional arguments :rel - the rel or the link in the JSON document. Required."},{"ref":"ExHal.Interpreter.html#extract/4","title":"ExHal.Interpreter.extract/4","module":"ExHal.Interpreter","type":"function","doc":""},{"ref":"ExHal.JsonFormEncoder.html","title":"ExHal.JsonFormEncoder","module":"ExHal.JsonFormEncoder","type":"module","doc":"Encodes forms into JSON documents."},{"ref":"ExHal.JsonFormEncoder.html#encode/1","title":"ExHal.JsonFormEncoder.encode/1","module":"ExHal.JsonFormEncoder","type":"function","doc":"Returns a string containing the JSON rendering of the provided form."},{"ref":"ExHal.Link.html","title":"ExHal.Link","module":"ExHal.Link","type":"module","doc":"A Link is a directed reference from one resource to another resource. They are found in the _links and _embedded sections of a HAL document"},{"ref":"ExHal.Link.html#embedded?/1","title":"ExHal.Link.embedded?/1","module":"ExHal.Link","type":"function","doc":""},{"ref":"ExHal.Link.html#embedded_link/0","title":"ExHal.Link.embedded_link/0","module":"ExHal.Link","type":"macro","doc":"Expands the embedded_link pattern. %{target: %{}} Binding Variables The following variables can be bound by giving them to embedded_link as keys on its last argument Keyword. For example: embedded_link(: x) Where x can be any value, variable in your scope or another pattern expansion. Not mentioned variables will be unbound and replaced by an _ at expansion site. Likewise, calling embedded_link() with no argumens will replace all its variables with _. Positional Variables embedded_link variables can also be bound by position, provided the last them is not a Keyword. For example: embedded_link(, bindings = []) Bang Constructor The embedded_link! constructor can be used to build data and make sure the guards are satisfied. Note that this macro can only be used as an expression and not as a matching pattern. For example: embedded_link!()"},{"ref":"ExHal.Link.html#embedded_link/1","title":"ExHal.Link.embedded_link/1","module":"ExHal.Link","type":"macro","doc":"Expands the embedded_link pattern. See embedded_link/0."},{"ref":"ExHal.Link.html#embedded_link!/0","title":"ExHal.Link.embedded_link!/0","module":"ExHal.Link","type":"macro","doc":"Builds data using the embedded_link pattern. See embedded_link/0."},{"ref":"ExHal.Link.html#equal?/2","title":"ExHal.Link.equal?/2","module":"ExHal.Link","type":"function","doc":"Returns true if the links are equivalent. Comparison rules: simple links are equal if their hrefs are equal and their names are equal. embedded links are equal if their hrefs are non-nil and equal a simple and an embedded link are equal if their hrefs are equal"},{"ref":"ExHal.Link.html#expand_curie/2","title":"ExHal.Link.expand_curie/2","module":"ExHal.Link","type":"function","doc":"Expands &quot;curie&quot;d link rels using the namespaces found in the curies link. Returns [%Link{}, ...] a link struct for each possible variation of the input link"},{"ref":"ExHal.Link.html#from_embedded/2","title":"ExHal.Link.from_embedded/2","module":"ExHal.Link","type":"function","doc":"Build new link struct from embedded doc."},{"ref":"ExHal.Link.html#from_links_entry/2","title":"ExHal.Link.from_links_entry/2","module":"ExHal.Link","type":"function","doc":"Build new link struct from _links entry."},{"ref":"ExHal.Link.html#simple_link/0","title":"ExHal.Link.simple_link/0","module":"ExHal.Link","type":"macro","doc":"Expands the simple_link pattern. %{target: nil} Binding Variables The following variables can be bound by giving them to simple_link as keys on its last argument Keyword. For example: simple_link(: x) Where x can be any value, variable in your scope or another pattern expansion. Not mentioned variables will be unbound and replaced by an _ at expansion site. Likewise, calling simple_link() with no argumens will replace all its variables with _. Positional Variables simple_link variables can also be bound by position, provided the last them is not a Keyword. For example: simple_link(, bindings = []) Bang Constructor The simple_link! constructor can be used to build data and make sure the guards are satisfied. Note that this macro can only be used as an expression and not as a matching pattern. For example: simple_link!()"},{"ref":"ExHal.Link.html#simple_link/1","title":"ExHal.Link.simple_link/1","module":"ExHal.Link","type":"macro","doc":"Expands the simple_link pattern. See simple_link/0."},{"ref":"ExHal.Link.html#simple_link!/0","title":"ExHal.Link.simple_link!/0","module":"ExHal.Link","type":"macro","doc":"Builds data using the simple_link pattern. See simple_link/0."},{"ref":"ExHal.Link.html#target_url/2","title":"ExHal.Link.target_url/2","module":"ExHal.Link","type":"function","doc":"Returns target url, expanded with vars if any are provided. Returns {:ok, &quot;fully_qualified_url&quot;} `:error` if link target is anonymous"},{"ref":"ExHal.Link.html#target_url!/2","title":"ExHal.Link.target_url!/2","module":"ExHal.Link","type":"function","doc":"Returns target url, expanded with vars if any are provided. Returns &quot;fully_qualified_url&quot; or raises exception"},{"ref":"ExHal.Link.html#to_json_hash/1","title":"ExHal.Link.to_json_hash/1","module":"ExHal.Link","type":"function","doc":"Deprecated See to_json_map/1"},{"ref":"ExHal.Link.html#to_json_map/1","title":"ExHal.Link.to_json_map/1","module":"ExHal.Link","type":"function","doc":"Returns a map that matches the shape of the intended JSON output."},{"ref":"ExHal.Link.html#unnamed_link/0","title":"ExHal.Link.unnamed_link/0","module":"ExHal.Link","type":"macro","doc":"Expands the unnamed_link pattern. %{name: nil} Binding Variables The following variables can be bound by giving them to unnamed_link as keys on its last argument Keyword. For example: unnamed_link(: x) Where x can be any value, variable in your scope or another pattern expansion. Not mentioned variables will be unbound and replaced by an _ at expansion site. Likewise, calling unnamed_link() with no argumens will replace all its variables with _. Positional Variables unnamed_link variables can also be bound by position, provided the last them is not a Keyword. For example: unnamed_link(, bindings = []) Bang Constructor The unnamed_link! constructor can be used to build data and make sure the guards are satisfied. Note that this macro can only be used as an expression and not as a matching pattern. For example: unnamed_link!()"},{"ref":"ExHal.Link.html#unnamed_link/1","title":"ExHal.Link.unnamed_link/1","module":"ExHal.Link","type":"macro","doc":"Expands the unnamed_link pattern. See unnamed_link/0."},{"ref":"ExHal.Link.html#unnamed_link!/0","title":"ExHal.Link.unnamed_link!/0","module":"ExHal.Link","type":"macro","doc":"Builds data using the unnamed_link pattern. See unnamed_link/0."},{"ref":"ExHal.Link.html#t:t/0","title":"ExHal.Link.t/0","module":"ExHal.Link","type":"type","doc":"A link. Links may be simple or dereferenced (from the embedded section)."},{"ref":"ExHal.Locatable.html","title":"ExHal.Locatable","module":"ExHal.Locatable","type":"protocol","doc":""},{"ref":"ExHal.Locatable.html#url/1","title":"ExHal.Locatable.url/1","module":"ExHal.Locatable","type":"function","doc":""},{"ref":"ExHal.Locatable.html#t:t/0","title":"ExHal.Locatable.t/0","module":"ExHal.Locatable","type":"type","doc":""},{"ref":"ExHal.Navigation.html","title":"ExHal.Navigation","module":"ExHal.Navigation","type":"module","doc":""},{"ref":"ExHal.Navigation.html#follow_link/3","title":"ExHal.Navigation.follow_link/3","module":"ExHal.Navigation","type":"function","doc":"Follows a link in a HAL document. Returns {:ok, %ExHal.Document{...}, %ExHal.ResponseHeader{...}} if response is successful; {:error, %ExHal.Error{...}} if not"},{"ref":"ExHal.Navigation.html#follow_links/2","title":"ExHal.Navigation.follow_links/2","module":"ExHal.Navigation","type":"function","doc":""},{"ref":"ExHal.Navigation.html#follow_links/3","title":"ExHal.Navigation.follow_links/3","module":"ExHal.Navigation","type":"function","doc":"Follows all links of a particular rel in a HAL document. Returns [{:ok, %ExHal.Document{...}, %ExHal.ResponseHeader{...}}, {:error, %ExHal.Error{...}, ...]"},{"ref":"ExHal.Navigation.html#follow_links/4","title":"ExHal.Navigation.follow_links/4","module":"ExHal.Navigation","type":"function","doc":""},{"ref":"ExHal.Navigation.html#link_target/3","title":"ExHal.Navigation.link_target/3","module":"ExHal.Navigation","type":"function","doc":"Returns {:ok, url} if a matching link is found or {:error, %ExHal.Error{...}} if not. a_doc - ExHal.Document in which to search for links name - the rel of the link of interest opts :tmpl_vars - Map of variables with which to expand any templates found. Default: %{} :strict - true if the existence of multiple matching links should cause a failure. Default: false"},{"ref":"ExHal.Navigation.html#link_target_lazy/4","title":"ExHal.Navigation.link_target_lazy/4","module":"ExHal.Navigation","type":"function","doc":""},{"ref":"ExHal.Navigation.html#link_targets/3","title":"ExHal.Navigation.link_targets/3","module":"ExHal.Navigation","type":"function","doc":"Returns {:ok, [url1, ...]} if a matching link is found or {:error, %ExHal.Error{...}} if not. a_doc - ExHal.Document in which to search for links name - the rel of the link of interest opts :tmpl_vars - Map of variables with which to expand any templates found. Default: %{}"},{"ref":"ExHal.Navigation.html#link_targets_lazy/4","title":"ExHal.Navigation.link_targets_lazy/4","module":"ExHal.Navigation","type":"function","doc":""},{"ref":"ExHal.Navigation.html#patch/4","title":"ExHal.Navigation.patch/4","module":"ExHal.Navigation","type":"function","doc":"PATCHs data to the named link in a HAL document. Returns {:ok, %ExHal.Document{...}, %ExHal.ResponseHeader{...}} if response is successful and body is HAL; {:error, %ExHal.Error{...}} if response is an error if not"},{"ref":"ExHal.Navigation.html#post/4","title":"ExHal.Navigation.post/4","module":"ExHal.Navigation","type":"function","doc":"Posts data to the named link in a HAL document. Returns {:ok, %ExHal.Document{...}, %ExHal.ResponseHeader{...}} if response is successful and body is HAL; {:error, %ExHal.Error{...}} if response is an error if not"},{"ref":"ExHal.Navigation.html#put/4","title":"ExHal.Navigation.put/4","module":"ExHal.Navigation","type":"function","doc":"PUTs data to the named link in a HAL document. Returns {:ok, %ExHal.Document{...}, %ExHal.ResponseHeader{...}} if response is successful and body is HAL; {:error, %ExHal.Error{...}} if response is an error if not"},{"ref":"ExHal.NonHalResponse.html","title":"ExHal.NonHalResponse","module":"ExHal.NonHalResponse","type":"module","doc":""},{"ref":"ExHal.NonHalResponse.html#from_httpoison_response/1","title":"ExHal.NonHalResponse.from_httpoison_response/1","module":"ExHal.NonHalResponse","type":"function","doc":""},{"ref":"ExHal.NsReg.html","title":"ExHal.NsReg","module":"ExHal.NsReg","type":"module","doc":"CURIE namespaces for a HAL document."},{"ref":"ExHal.NsReg.html#from_parsed_json/1","title":"ExHal.NsReg.from_parsed_json/1","module":"ExHal.NsReg","type":"function","doc":"Returns %ExHal.NsReg{}"},{"ref":"ExHal.NsReg.html#variations/2","title":"ExHal.NsReg.variations/2","module":"ExHal.NsReg","type":"function","doc":"Returns list of all valid variations of the specified rel."},{"ref":"ExHal.ResponseHeader.html","title":"ExHal.ResponseHeader","module":"ExHal.ResponseHeader","type":"module","doc":"Map that stores HTTP response header information"},{"ref":"ExHal.Transcoder.html","title":"ExHal.Transcoder","module":"ExHal.Transcoder","type":"behaviour","doc":"Helps to build transcoders for HAL documents. Given a document like { &quot;name&quot;: &quot;Jane Doe&quot;, &quot;mailingAddress&quot;: &quot;123 Main St&quot;, &quot;_links&quot;: { &quot;app:department&quot;: { &quot;href&quot;: &quot;http://example.com/dept/42&quot; }, &quot;app:manager&quot;: { &quot;href&quot;: &quot;http://example.com/people/84&quot; } } } We can define an transcoder for it. defmodule PersonTranscoder do use ExHal.Transcoder defproperty &quot;name&quot; defproperty &quot;mailingAddress&quot;, param: :address deflink &quot;app:department&quot;, param: :department_url deflink &quot;app:manager&quot;, param: :manager_id, value_converter: PersonUrlConverter end PersonUrlConverter is a module that has adopted the ExHal.ValueConverter behavior. defmodule PersonUrlConverter do @behaviour ExHal.ValueConveter def from_hal(person_url) do to_string(person_url) |&gt; String.split(&quot;/&quot;) |&gt; List.last end def to_hal(person_id) do &quot;http://example.com/people/\#{person_id}&quot; end end We can use this transcoder to to extract the pertinent parts of the document into a map. iex&gt; PersonTranscoder.decode!(doc) %{name: &quot;Jane Doe&quot;, address: &quot;123 Main St&quot;, department_url: &quot;http://example.com/dept/42&quot;, manager_id: 84} iex&gt; PersonTranscoder.encode!(%{name: &quot;Jane Doe&quot;, address: &quot;123 Main St&quot;, department_url: &quot;http://example.com/dept/42&quot;, manager_id: 84}) ~s( { &quot;name&quot;: &quot;Jane Doe&quot;, &quot;mailingAddress&quot;: &quot;123 Main St&quot;, &quot;_links&quot;: { &quot;app:department&quot;: { &quot;href&quot;: &quot;http://example.com/dept/42&quot; }, &quot;app:manager&quot;: { &quot;href&quot;: &quot;http://example.com/people/84&quot; } } } )"},{"ref":"ExHal.Transcoder.html#c:decode!/1","title":"ExHal.Transcoder.decode!/1","module":"ExHal.Transcoder","type":"callback","doc":"Returns a decoded version of HAL document merged with the initial params. initial_params - the initial params with which the newly extracted info should merged. src_doc - the document to interpret opts - options for use by modules adopting ExHal.ValueConverterWithOptions behaviour"},{"ref":"ExHal.Transcoder.html#c:decode!/2","title":"ExHal.Transcoder.decode!/2","module":"ExHal.Transcoder","type":"callback","doc":""},{"ref":"ExHal.Transcoder.html#c:decode!/2","title":"ExHal.Transcoder.decode!/2","module":"ExHal.Transcoder","type":"callback","doc":""},{"ref":"ExHal.Transcoder.html#c:decode!/3","title":"ExHal.Transcoder.decode!/3","module":"ExHal.Transcoder","type":"callback","doc":""},{"ref":"ExHal.Transcoder.html#decode_value/2","title":"ExHal.Transcoder.decode_value/2","module":"ExHal.Transcoder","type":"function","doc":""},{"ref":"ExHal.Transcoder.html#decode_value/3","title":"ExHal.Transcoder.decode_value/3","module":"ExHal.Transcoder","type":"function","doc":""},{"ref":"ExHal.Transcoder.html#deflink/2","title":"ExHal.Transcoder.deflink/2","module":"ExHal.Transcoder","type":"macro","doc":"Define a link extractor &amp; injector. rel - the rel of the link in HAL options - Keywords arguments :param - the key(s) in the param structure that maps to this link. Required. :templated - a boolean that adds a templated: true parameter if true :value_converter - a ExHal.Transcoder.ValueConverter with which to convert the link target when en/decoding HAL"},{"ref":"ExHal.Transcoder.html#deflinks/2","title":"ExHal.Transcoder.deflinks/2","module":"ExHal.Transcoder","type":"macro","doc":"Define a link extractor &amp; injector for links that may have more than one item. rel - the rel of the link in HAL options - Keywords arguments :param - the key(s) in the param structure that maps to this link. Required. :value_converter - a ExHal.Transcoder.ValueConverter with which to convert the link target when en/decoding HAL"},{"ref":"ExHal.Transcoder.html#defproperty/2","title":"ExHal.Transcoder.defproperty/2","module":"ExHal.Transcoder","type":"macro","doc":"Define a property extractor and injector. name - the name of the property in HAL options - Keywords arguments :param - the key(s) in the param structure that map to this property. Default is String.to_atom(name). :value_converter - a ExHal.Transcoder.ValueConverter with which to convert the value to and from HAL"},{"ref":"ExHal.Transcoder.html#c:encode!/1","title":"ExHal.Transcoder.encode!/1","module":"ExHal.Transcoder","type":"callback","doc":"Returns an HAL version of params provided, combined with the initial doc. initial_doc - the initial document with which the newly encoded info should merged. src_params - the params to encoded into HAL opts - options for use by modules adopting ExHal.ValueConverterWithOptions behaviour"},{"ref":"ExHal.Transcoder.html#c:encode!/2","title":"ExHal.Transcoder.encode!/2","module":"ExHal.Transcoder","type":"callback","doc":""},{"ref":"ExHal.Transcoder.html#c:encode!/2","title":"ExHal.Transcoder.encode!/2","module":"ExHal.Transcoder","type":"callback","doc":""},{"ref":"ExHal.Transcoder.html#c:encode!/3","title":"ExHal.Transcoder.encode!/3","module":"ExHal.Transcoder","type":"callback","doc":""},{"ref":"ExHal.Transcoder.html#encode_value/3","title":"ExHal.Transcoder.encode_value/3","module":"ExHal.Transcoder","type":"function","doc":""},{"ref":"ExHal.Transcoder.html#c:patch!/2","title":"ExHal.Transcoder.patch!/2","module":"ExHal.Transcoder","type":"callback","doc":"Updates an existing object, such as one created by ExHal.Transcoder.decode! initial_object - a map containing properties, links etc patch_ops - a list of JSON-patch operations (https://tools.ietf.org/html/rfc6902) Supported are &quot;replace&quot; for properties, links, and collections of links and &quot;add&quot; for collections of links. For example: [ %{&quot;op =&gt; &quot;replace&quot;, &quot;path&quot; =&gt; &quot;/panicLevel&quot;, &quot;value&quot; =&gt; 42}, # replace a property %{&quot;op =&gt; &quot;replace, &quot;path&quot; =&gt; &quot;/_links/answers&quot;, &quot;value&quot; =&gt; [ &quot;urn:earth&quot;, &quot;urn:universe&quot;]} # replace link collection %{&quot;op =&gt; &quot;add&quot;, &quot;path&quot; =&gt; &quot;/_links/answers/-&quot;, &quot;value&quot; =&gt; &quot;urn:everyhing&quot;}] # append to link collection"},{"ref":"ExHal.Transcoder.html#c:patch!/3","title":"ExHal.Transcoder.patch!/3","module":"ExHal.Transcoder","type":"callback","doc":""},{"ref":"ExHal.Transcoder.html#put_link/3","title":"ExHal.Transcoder.put_link/3","module":"ExHal.Transcoder","type":"function","doc":""},{"ref":"ExHal.Transcoder.html#put_link/4","title":"ExHal.Transcoder.put_link/4","module":"ExHal.Transcoder","type":"function","doc":""},{"ref":"ExHal.Transcoder.html#put_param/3","title":"ExHal.Transcoder.put_param/3","module":"ExHal.Transcoder","type":"function","doc":""},{"ref":"ExHal.Transcoder.html#put_property/3","title":"ExHal.Transcoder.put_property/3","module":"ExHal.Transcoder","type":"function","doc":""},{"ref":"ExHal.Transcoder.html#t:t/0","title":"ExHal.Transcoder.t/0","module":"ExHal.Transcoder","type":"type","doc":""},{"ref":"ExHal.Transcoder.IdentityConverter.html","title":"ExHal.Transcoder.IdentityConverter","module":"ExHal.Transcoder.IdentityConverter","type":"module","doc":""},{"ref":"ExHal.Transcoder.IdentityConverter.html#from_hal/1","title":"ExHal.Transcoder.IdentityConverter.from_hal/1","module":"ExHal.Transcoder.IdentityConverter","type":"function","doc":"Returns Elixir representation of HAL value. hal_value - The HAL representation of the value to convert. Callback implementation for ExHal.Transcoder.ValueConverter.from_hal/1."},{"ref":"ExHal.Transcoder.IdentityConverter.html#to_hal/1","title":"ExHal.Transcoder.IdentityConverter.to_hal/1","module":"ExHal.Transcoder.IdentityConverter","type":"function","doc":"Returns HAL representation of Elixir value. elixir_value - The Elixir representation of the value to convert. Callback implementation for ExHal.Transcoder.ValueConverter.to_hal/1."},{"ref":"ExHal.Transcoder.ValueConverter.html","title":"ExHal.Transcoder.ValueConverter","module":"ExHal.Transcoder.ValueConverter","type":"behaviour","doc":""},{"ref":"ExHal.Transcoder.ValueConverter.html#c:from_hal/1","title":"ExHal.Transcoder.ValueConverter.from_hal/1","module":"ExHal.Transcoder.ValueConverter","type":"callback","doc":"Returns Elixir representation of HAL value. hal_value - The HAL representation of the value to convert."},{"ref":"ExHal.Transcoder.ValueConverter.html#c:to_hal/1","title":"ExHal.Transcoder.ValueConverter.to_hal/1","module":"ExHal.Transcoder.ValueConverter","type":"callback","doc":"Returns HAL representation of Elixir value. elixir_value - The Elixir representation of the value to convert."},{"ref":"ExHal.Transcoder.ValueConverter.html#t:t/0","title":"ExHal.Transcoder.ValueConverter.t/0","module":"ExHal.Transcoder.ValueConverter","type":"type","doc":""},{"ref":"ExHal.Transcoder.ValueConverterWithOptions.html","title":"ExHal.Transcoder.ValueConverterWithOptions","module":"ExHal.Transcoder.ValueConverterWithOptions","type":"behaviour","doc":""},{"ref":"ExHal.Transcoder.ValueConverterWithOptions.html#c:from_hal/2","title":"ExHal.Transcoder.ValueConverterWithOptions.from_hal/2","module":"ExHal.Transcoder.ValueConverterWithOptions","type":"callback","doc":"Returns Elixir representation of HAL value. hal_value - The HAL representation of the value to convert. opts - Options to be used by the converter."},{"ref":"ExHal.Transcoder.ValueConverterWithOptions.html#c:to_hal/2","title":"ExHal.Transcoder.ValueConverterWithOptions.to_hal/2","module":"ExHal.Transcoder.ValueConverterWithOptions","type":"callback","doc":"Returns HAL representation of Elixir value. elixir_value - The Elixir representation of the value to convert. opts - Options to be used by the converter."},{"ref":"ExHal.Transcoder.ValueConverterWithOptions.html#t:t/0","title":"ExHal.Transcoder.ValueConverterWithOptions.t/0","module":"ExHal.Transcoder.ValueConverterWithOptions","type":"type","doc":""},{"ref":"Mix.Tasks.Release.html","title":"Mix.Tasks.Release","module":"Mix.Tasks.Release","type":"task","doc":""},{"ref":"Mix.Tasks.Release.html#run/1","title":"Mix.Tasks.Release.run/1","module":"Mix.Tasks.Release","type":"function","doc":"A task needs to implement run which receives a list of command line args. Callback implementation for Mix.Task.run/1."}]